package perl5;

/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version: 1.3.21
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */


public class PLJava {
  public static int PLJava_start() {
    return PLJavaJNI.PLJava_start();
  }

  public static String PLJava_eval(String code) {
    return PLJavaJNI.PLJava_eval(code);
  }

  public static String PLJava_eval_sv(String code) {
    return PLJavaJNI.PLJava_eval_sv(code);
  }

  public static String PLJava_error() {
    return PLJavaJNI.PLJava_error();
  }

  public static void PLJava_stop() {
    PLJavaJNI.PLJava_stop();
  }

}
