package perl5;

/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version: 1.3.21
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */


class PLJavaJNI {
  public final static native int PLJava_start();
  public final static native String PLJava_eval(String jarg1);
  public final static native String PLJava_eval_sv(String jarg1);
  public final static native String PLJava_error();
  public final static native void PLJava_stop();
}
